
# This script organizes the Quaterly WEI+ data at subbasin level that will be used for the analysis
# The excel sheets obtained from EEA are transformed to multi-dimensional array for analysis

library(readxl) # Reading spreadsheet
library(sf)
library(ggplot2)
library(ggspatial)
library(dplyr)
library(colorspace)
library(tidyr)

path.intermediate_data <- 'folder/'

# Fetch world map data
library(rnaturalearth)
library(rnaturalearthdata)

world_map <- ne_countries(scale = "medium", returnclass = "sf")

regions <-  'EEA_riverbasin'

# WEI+ 1990-2015

data_1990_q1 <- read_excel(paste0(path.intermediate_data, "exposure_data/original_wei_1990_2015/SB_WEI_MAA_Q1_v4.xlsx"))
data_1990_q2 <- read_excel(paste0(path.intermediate_data, "exposure_data/original_wei_1990_2015/SB_WEI_MAA_Q2_v4.xlsx"))
data_1990_q3 <- read_excel(paste0(path.intermediate_data, "exposure_data/original_wei_1990_2015/SB_WEI_MAA_Q3_v4.xlsx"))
data_1990_q4 <- read_excel(paste0(path.intermediate_data, "exposure_data/original_wei_1990_2015/SB_WEI_MAA_Q4_v4.xlsx"))

data_1990_q1 <- arrange(data_1990_q1, Entity)
data_1990_q2 <- arrange(data_1990_q2, Entity)
data_1990_q3 <- arrange(data_1990_q3, Entity)
data_1990_q4 <- arrange(data_1990_q4, Entity)

# region_code_1990_q1 <- unique(data_1990_q1[['Entity']])
# region_code_1990_q2 <- unique(data_1990_q2[['Entity']])
# region_code_1990_q3 <- unique(data_1990_q3[['Entity']])
# region_code_1990_q4 <- unique(data_1990_q4[['Entity']])

data_1990 <- rbind(data_1990_q1, data_1990_q2, data_1990_q3, data_1990_q4)
data_1990 <- data_1990[, c("Entity", "Year", "Season", "Water exploitation index plus")]
colnames(data_1990) <- c("Entity", "Year", "Season", "WEI")

column <- data_1990[['Entity']]
region_code_1990 <- unique(column)
n_reg_1990 <- length(region_code_1990)

region_code_1990_sorted <- sort(region_code_1990)

# WEI+ 2016-2019

data_2016 <- read_excel(paste0(path.intermediate_data, 'exposure_data/WEI_river_sub-basin_2016-2019.xlsx', sheet = 'DATASET_for_Mapping'))
column <- data_2016[['Entity']]
region_code_2016 <- unique(column)
n_reg_2016 <- length(region_code_2016)

# # Sort the data frames by Entity code
# entity_basin_1990 <- entity_basin_1990[order(entity_basin_1990$Entity), ]
# entity_basin_2016 <- entity_basin_2016[order(entity_basin_2016$Entity), ]

# Common sub-basin level data - All the regions files are read
common_codes <- union(region_code_2016, region_code_1990_sorted)

# Common sub-basin for all_data_available_1990_2019 - Uncomment the below lines for that
# common_codes <- intersect(region_code_2016, region_code_1990_sorted)
# saveRDS(common_codes, file=paste('/esarchive/scratch/bsolaraj/LANCET_2023_final/rdata/exposure_data/common_codes_all_data_available_1990_2019_',regions,'.Rds', sep=""))

# common_codes <- intersect(region_code_2016, region_code_1990_sorted)
# setdiff(region_code_2016, region_code_1990_sorted)
count <- length(common_codes)

# Find common regions
common_regions_1990 <- data_1990[data_1990$'Entity' %in% common_codes, ]
common_regions_2016 <- data_2016[data_2016$'Entity' %in% common_codes, ]

# Order the data based on the common codes, years and season
common_regions_1990 <- common_regions_1990 %>%  arrange(match(Entity, common_codes))
common_regions_1990_extract <- common_regions_1990[, c("Entity", "Year", "Season", "WEI")]

common_regions_2016 <- common_regions_2016 %>%  arrange(match(Entity, common_codes))
common_regions_2016_extract <- common_regions_2016[, c("Entity", "Year", "Season", "WEI")]

# Combine both dataset now

common_regions_1990_2019 <- rbind(common_regions_1990_extract, common_regions_2016_extract)

# x <- as.vector(unique(common_regions_1990_extract[,1]))
# nrow(x)
# x <- as.vector(unique(common_regions_2016_extract[,1]))
# nrow(x)
# x <- as.vector(unique(common_regions_1990_2019[,1]))
# nrow(x)

# WEI values arranged

wei_array <- common_regions_1990_2019 %>%
  select(Entity, Year, Season, WEI) %>%
  distinct() %>%
  group_by(Entity, Year, Season) %>%
  summarise(WEI_values = list(WEI)) %>%
  ungroup() %>%
  unnest(WEI_values)


# Dimensions
num_entity <- length(unique(wei_array[['Entity']]))
num_years <- length(unique(wei_array[['Year']]))
num_seasons <- length(unique(wei_array[['Season']]))
# num_wei_values <- length(unique(wei_array[['WEI_values']]))

# Create a data frame with all possible combinations of Entity, Year, and Season
all_combinations <- expand.grid(
  Entity = unique(wei_array$Entity),
  Year = unique(wei_array$Year),
  Season = unique(wei_array$Season)
)

# Check for missing combinations in wei_array
missing_combinations <- anti_join(all_combinations, wei_array, by = c("Entity", "Year", "Season"))

# Add NA values to missing_combinations
missing_combinations$WEI_values <- NA

# Combine the missing_combinations and wei_array data frames
all_data <- rbind(wei_array, missing_combinations)
wei_array <- all_data

# Creating the array
array <- array(dim = c(num_entity, num_years, num_seasons))
for (i in 1:num_entity) {
  for (j in 1:num_years) {
    for (k in 1:num_seasons) {
      entity <- unique(wei_array[['Entity']])[i]
      year <- unique(wei_array[['Year']])[j]
      season <- unique(wei_array[['Season']])[k]
      
      value <- wei_array %>%
        filter(Entity == entity, Year == year, Season == season) %>%
        pull(WEI_values)
      
      array[i, j, k] <- value
    }
  }
}

names(dim(array)) <- c('Entity','Year','Season')

data_array <- list()
data_array$array <- array
data_array$Entity <- unique(wei_array$Entity)
data_array$Year <- unique(wei_array$Year)
data_array$Season <- unique(wei_array$Season)

saveRDS(data_array, file=paste(paste0(path.intermediate_data, 'rdata/exposure_data/data_wei_',regions,'.Rds', sep="")))
